//  2021, CGVR (https://cgvr.informatik.uni-bremen.de/), Andre Mhlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "CoreMinimal.h"

#include "GridDetector.h"

/**
 * 
 */
class VRKINECTREGISTRATION_API MultiplePointCloudsRegistrator
{
private:
	int pointCloudCount;
	TArray<TArray<FVector>> correspondences;
	TArray<float> meanErrors;

	TArray<int> lastUsedCorrespondencePointSize;
	TArray<int> lastTotalCorrespondencePointSize;

	float meanError = 0;

public:
	MultiplePointCloudsRegistrator();
	MultiplePointCloudsRegistrator(unsigned int pointCloudCount);
	~MultiplePointCloudsRegistrator();

	unsigned int getPointCloudCount() {
		return pointCloudCount;
	}
	
	/**
	 * Returns the mean error after registration between the point cloud with the given index and the 
	 * reference point cloud chosen in the registerInto-function.
	 */

	float getMeanError(int pointCloudIndex);

	/**
	* Adds correspondence points of multiple pointclouds. 
	*
	* NOTE: Please ensure that all arrays of FVectors will have the same size. If a point 
	* was not recognized by a single point cloud, simply set the value of the FVector to 
	* NAN - in this case, this point will not be considered for the registration of the 
	* point cloud.
	*/

	void addCorrespondencePoints(TArray<TArray<FVector>> points);

	/**
	* Register all the point clouds into the pointcloud of the given pointCloudID.
	*
	* The transform with the id of the given point cloud id will be the identity.
	*/

	TArray<FTransform> registerInto(int pointCloudID, float correspondenceRejectionThreshold = 0.04f);

	/**
	* Get the number of correspondence points used for the last registration call
	* for the given index of the point cloud to be registered
	*/

	int getLastUsedCorrespondencePointSize(int index) {
		return lastUsedCorrespondencePointSize[index];
	}

	/**
	* Get the total number of correspondence points in the last registration call
	* for the given index of the point cloud to be registered.
	*/

	int getLastTotalCorrespondencePointSize(int index) {
		return lastTotalCorrespondencePointSize[index];
	}
};
