#pragma once

#include "CoreMinimal.h"

/*
* Result of a grid detection in a single point cloud.
*/

struct GridDetectionResult {
	bool gridFound = false;

	// Mid points of the grid holes (not sorted yet!):
	TArray<FVector> centersOfFoundHoles;

	// Location of the grid center:
	FVector gridCenter;

	// X-Axis of the grid:
	FVector axis1;

	// Y-Axis of the grid:
	FVector axis2;

	// Z-Axis of the grid:
	FVector normal;

	// Vector from the grid center to the hand
	FVector vectorToHandsMidpoint;
};

/*
* Defines a point cloud which still is ordered along the 2D image.
*/

struct PointCloudImage {
	/* Point Cloud with x,y and z values for each point */
	float* data;

	/* Width of the original depth image */
	unsigned int sourceImageWidth;

	/* Height of the original depth image */
	unsigned int sourceImageHeight;

	PointCloudImage(float* pData, unsigned int pSourceImageWidth, unsigned int pSourceImageHeight)
		: data(pData)
		, sourceImageWidth(pSourceImageWidth)
		, sourceImageHeight(pSourceImageHeight)
	{}
};

/*
* Assumptions about the motion controller where they are relative to the grid.
*/

struct MotionControllerAssumptions {
	bool swapControllers = true;

	FVector leftOffsetFromGridMid;
	FVector rightOffsetFromGridMid;
};

/*
* The result, where the motion controllers are relative to the camera coordinate system
* which recorded the point cloud.
*/

struct MotionControllerResult {
	bool found = false;

	FVector leftControllerPosition;
	FVector rightControllerPosition;
};