// Copyright Epic Games, Inc. All Rights Reserved.

using System.IO;
using UnrealBuildTool;

public class VRKinectRegistration : ModuleRules
{
	private string ThirdPartyPath
	{
		get { return Path.Combine(ModuleDirectory, "../../ThirdParty"); }
	}

	private string BinariesPath
	{
		get { return Path.GetFullPath(Path.Combine(ThirdPartyPath, "../Binaries/")); }
	}
	
	public VRKinectRegistration(ReadOnlyTargetRules Target) : base(Target)
	{
		PCHUsage = ModuleRules.PCHUsageMode.UseExplicitOrSharedPCHs;
		
		PublicIncludePaths.AddRange(
			new string[] {
				// ... add public include paths required here ...
			}
			);
				
		
		PrivateIncludePaths.AddRange(
			new string[] {
				// ... add other private include paths required here ...
			}
			);
			
		
		PublicDependencyModuleNames.AddRange(
			new string[]
			{
				"Core",
                "Projects"
				// ... add other public dependencies that you statically link with here ...
			}
			);
			
		
		PrivateDependencyModuleNames.AddRange(
			new string[]
			{
				 "CoreUObject", 
				 "Engine", 
				 "Slate", 
				 "SlateCore"
				// ... add private dependencies that you statically link with here ...	
			}
			);
		
		
		DynamicallyLoadedModuleNames.AddRange(
			new string[]
			{
				// ... add any modules that your module loads dynamically here ...
			}
			);

        LoadPCL(Target);
    }

    public bool LoadPCL(ReadOnlyTargetRules Target)
    {
        bool isLibrarySupported = false;
        //bool bDebug = (Target.Configuration == UnrealTargetConfiguration.Debug && BuildConfiguration.bDebugBuildsActuallyUseDebugCRT);

        if (Target.Platform == UnrealTargetPlatform.Win64)
        {
            isLibrarySupported = true;

            //string PlatformString = (Target.Platform == UnrealTargetPlatform.Win64) ? "x64" : "x86";

            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_chrono-vc141-mt-x64-1_68.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_date_time-vc141-mt-x64-1_68.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_filesystem-vc141-mt-x64-1_68.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_iostreams-vc141-mt-x64-1_68.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_system-vc141-mt-x64-1_68.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_thread-vc141-mt-x64-1_68.lib"));

            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_common_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_io_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_io_ply_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_kdtree_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_octree_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_ml_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/lib/pcl_sample_consensus_release.lib"));

            //RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_common_release.dll");
            // RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_io_release.dll");
            // RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_io_ply_release.dll");
            // RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_octree_release.dll");
            // RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_ml_release.dll");
            // RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_sample_consensus_release.dll");

            // Explicitly name the used libraries
            /*
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_chrono-vc141-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_date_time-vc141-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_filesystem-vc141-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_iostreams-vc141-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_system-vc141-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "Boost/lib/libboost_thread-vc141-mt-1_64.lib"));

            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_common_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_io_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_io_ply_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_octree_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_ml_release.lib"));
            PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_sample_consensus_release.lib"));
            //PublicAdditionalLibraries.Add(Path.Combine(ThirdPartyPath, "PCL_181/lib/pcl_visualization_release.lib"));

            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_common_release.dll");
            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_io_release.dll");
            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_io_ply_release.dll");
            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_octree_release.dll");
            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_ml_release.dll");
            RuntimeDependencies.Add("$(PluginDir)/ThirdParty/PCL_181/bin/pcl_sample_consensus_release.dll");
            */
        }

        if (isLibrarySupported)
        {

            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "Eigen/eigen3"));
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "Boost/include/boost-1_68"));
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "PCL_1_9_1/include/pcl-1.9"));
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "FLANN/include"));
            /*
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "Eigen/eigen3"));
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "Boost/include/boost-1_64"));
            PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "PCL_181/include/pcl-1.8"));
            //PublicIncludePaths.Add(Path.Combine(ThirdPartyPath, "VTK/include/vtk-8.0"));
            */

            // Not sure if needed
            PublicDefinitions.Add("BOOST_DISABLE_ABI_HEADERS=1");

            // Needed configurations in order to run Boost
            bUseRTTI = true;
            bEnableExceptions = true;
            bLegacyPublicIncludePaths = false;

            //bEnableUndefinedIdentifierWarnings = false;
        }

        PublicDefinitions.Add(string.Format("WITH_PCL_BINDING={0}", isLibrarySupported ? 1 : 0));
        PublicDefinitions.Add(string.Format("WITH_BOOST_BINDING={0}", isLibrarySupported ? 1 : 0));
        PublicDefinitions.Add("OPENMP_AVAILABLE_RANSAC=true");

        return isLibrarySupported;
    }
}
