// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"

#include "k4a/k4a.hpp"
#include "k4abt.hpp"
#include "PointCloudRenderer.h"
#include "AzureKinectThread.h"

#include "GridDetector.h"

#include "AzureKinect.generated.h"


UCLASS()
class VRKINECTREG_API AAzureKinect : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	AAzureKinect();
	~AAzureKinect();

	bool Init(int deviceID);
	bool Shutdown();

	bool IsInitialized();

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Point Cloud")
		UPointCloudRenderer* pcRendererComponent;

	UStaticMeshComponent* gridMeshComponent;
	UStaticMeshComponent* estimatedLeftController;
	UStaticMeshComponent* estimatedRightController;

	bool ReadImage();

	void CreateXYLookupTable();

	float* currentTmpPointCloudData() {
		return tmpPointCloud;
	}

	unsigned int getWidth() {
		return width;
	}

	unsigned int getHeight() {
		return height;
	}

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

private:
	/** native handle to kinect device */
	k4a::device nativeKinectDevice;
	k4a::calibration sensorCalibration;
	k4a::transformation deviceTransform;
	k4abt::tracker bodyTracker;

	k4a::image xyLookupTable = NULL;

	bool bIsInitialized = false;
	int iDeviceID = -1;

	float* tmpPointCloud = nullptr;
	uint8_t* tmpColorImage = nullptr;

	unsigned int width;
	unsigned int height;

	FAzureKinectThread* thread = nullptr;

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

};
