// Fill out your copyright notice in the Description page of Project Settings.


#include "AzureKinectThread.h"

#include "HAL/PlatformProcess.h"
#include "AzureKinect.h"


FAzureKinectThread::FAzureKinectThread(AAzureKinect* kinect) :
	thread(nullptr),
	stopCounter(0),
	kinect(kinect)
{
	thread = FRunnableThread::Create(this, TEXT("FAzureKinectThread"), 0, TPri_AboveNormal);
}

FAzureKinectThread::~FAzureKinectThread()
{
	if (thread)
	{
		delete thread;
		thread = nullptr;
	}
}


void FAzureKinectThread::Terminate()
{
	Stop();
	if (thread)
	{
		thread->WaitForCompletion();
	}
}


bool FAzureKinectThread::Init()
{
	UE_LOG(LogTemp, Log, TEXT("Azure Kinect thread started."));
	return true;
}

uint32 FAzureKinectThread::Run()
{
	if (!kinect)
		return 1;

	while (stopCounter.GetValue() == 0)
	{
		kinect->ReadImage();
	}

	return 0;
}

void FAzureKinectThread::Stop()
{
	stopCounter.Increment();
}
