// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"

#include "NiagaraComponent.h"
#include "NiagaraSystem.h"
#include "NiagaraFunctionLibrary.h"
#include "NiagaraDataInterfaceTexture.h"
#include "Engine/Texture2D.h"

#include "PointCloudRenderer.generated.h"

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class VRKINECTREG_API UPointCloudRenderer : public USceneComponent
{
	GENERATED_BODY()

public:	
	UPointCloudRenderer();

	bool Init(float* pointCloudArray, uint8_t* colorImageArray, int tWidth, int tHeight, int KinectID);

	/* This variable has to be defined before rendering starts! */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PC")
		UNiagaraComponent* rendererInstance;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PC")
		bool useMonotonicColor = false;

	int deviceID = -1;
	uint8_t defaultPCColor[3];

	bool IsInitialized() {
		return bIsInitialized;
	}


protected:
	// Called when the game starts
	virtual void BeginPlay() override;

private:
	// The point cloud data (it is expected that it's content is created and changed
	// somewhere else:
	float* pointCloudData;
	uint8_t* colorImageData;

	// Textures to transfer the data to the niagara system:
	UTexture2D* positionTexture;
	UTexture2D* colorTexture;

	// further data:
	uint32_t textureWidth;
	uint32_t textureHeight;
	uint32_t pointCount;

	UNiagaraSystem* pointCloudRenderer;

	// Runtime of the game:
	float runtime;

	// Definition of the region which should be updated:
	FUpdateTextureRegion2D region;

	bool bIsInitialized = false;

public:	
	// Called every frame
	virtual void TickComponent(float DeltaTime, ELevelTick TickType, FActorComponentTickFunction* ThisTickFunction) override;

		
};
