// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"

#include "AzureKinect.h"
#include "RegistrationThread.h"

#include "GridDetector.h"
#include "MultiplePointCloudsRegistrator.h"
#include "CorrespondencePointMatcher.h"

#include "RegistrationManager.generated.h"

UCLASS()
class VRKINECTREG_API ARegistrationManager : public AActor
{
	GENERATED_BODY()
	
public:	
	// Sets default values for this actor's properties
	ARegistrationManager();

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		bool shouldRegisterMultipleKinects = false;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		bool shouldRegisterKinectIntoHMD = false;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		FVector leftControllerOffset = FVector(0.24f, 0.24f, 0);

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		FVector rightControllerOffset = FVector(0.24f, -0.24f, 0);

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		float correspondenceRejectionKinectsThreshold = 0.04f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		float correspondenceRejectionToWorldThreshold = 0.08f;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "Registration")
		bool swapControllers = false;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Registration")
		float meanKinect0ToWorldError = 0.f;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Registration")
		int correspondenceCountVRTotal = 0.f;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Registration")
		int correspondenceCountVRUsed = 0.f;
		

	UFUNCTION(BlueprintCallable)
		void UpdateMotionControllerPositions(FVector left, FVector right);

	void RegisterOnCurrent();
	void RegisterMultipleKinects();
	void RegisterKinectsIntoHMD();

	UStaticMeshComponent* gridMeshComponent;
protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;


private:
	TArray<AAzureKinect*> kinects;

	MultiplePointCloudsRegistrator pcWithHMDRegistrator;
	MultiplePointCloudsRegistrator roughPCRegistrator;
	MultiplePointCloudsRegistrator multiplePCRegistrator;
	TArray<FTransform> currentRegistration;

	FTransform kinect0IntoWorldRegistration;
	
	FRegistrationThread* thread = nullptr;

	FVector leftControllerPosition;
	FVector rightControllerPosition;
	bool controllerPositionsAvailable = false;

	FTransform debugDrawGridTransform;
	FVector debugDrawLeftControllerCS;
	FVector debugDrawRightControllerCS;

	int tickCounter = 0;
};