// Fill out your copyright notice in the Description page of Project Settings.


#include "RegistrationThread.h"

#include "HAL/PlatformProcess.h"
#include "RegistrationManager.h"


FRegistrationThread::FRegistrationThread(ARegistrationManager *registrationManager) :
	thread(nullptr),
	stopCounter(0),
	registrationManager(registrationManager)
{
	thread = FRunnableThread::Create(this, TEXT("FRegistrationThread"), 0, TPri_AboveNormal);
}

FRegistrationThread::~FRegistrationThread()
{
	if (thread)
	{
		delete thread;
		thread = nullptr;
	}
}


void FRegistrationThread::Terminate()
{
	Stop();
	if (thread)
	{
		thread->WaitForCompletion();
	}
}


bool FRegistrationThread::Init()
{
	UE_LOG(LogTemp, Log, TEXT("Azure Kinect thread started."));
	return true;
}

uint32 FRegistrationThread::Run()
{
	if (!registrationManager)
		return 1;

	while (stopCounter.GetValue() == 0)
	{
		// Perform registration:
		registrationManager->RegisterOnCurrent();
		// Use at maximum 10 images per second:
		FPlatformProcess::Sleep(0.1);
	}

	return 0;
}

void FRegistrationThread::Stop()
{
	stopCounter.Increment();
}
