// Copyright Epic Games, Inc. All Rights Reserved.

using System.IO;
using UnrealBuildTool;

public class VRKinectReg : ModuleRules
{
    private string ThirdPartyPath
    {
        get { return Path.Combine(ModuleDirectory, "../../ThirdParty"); }
    }

    private string BinariesPath
    {
        get { return Path.GetFullPath(Path.Combine(ThirdPartyPath, "../Binaries/")); }
    }

    public VRKinectReg(ReadOnlyTargetRules Target) : base(Target)
	{
		PCHUsage = PCHUsageMode.UseExplicitOrSharedPCHs;

		PublicDependencyModuleNames.AddRange(new string[] {
            "Core",
            "CoreUObject",
            "Engine",
            "InputCore",
            "HeadMountedDisplay",
            "Niagara",
            "VRKinectRegistration"
        });

        LoadAzureSDKs(Target);
    }

    public bool LoadAzureSDKs(ReadOnlyTargetRules Target)
    {

        string azureKinectSensorSDKPath = Path.Combine(ThirdPartyPath, "AzureKinectSensor");
        string azureKinectBodyTrackingSDKPath = Path.Combine(ThirdPartyPath, "AzureKinectBodyTracking");

        PublicIncludePaths.AddRange(
            new string[]
            {
                Path.Combine(azureKinectSensorSDKPath, "include"),
                Path.Combine(azureKinectBodyTrackingSDKPath, "include")
            });

        // Libs path
        //PublicLibraryPaths.Add(Path.Combine(azureKinectSensorSDKPath, "amd64", "lib"));
        //PublicLibraryPaths.Add(Path.Combine(azureKinectBodyTrackingSDKPath, "amd64", "lib"));

        PublicAdditionalLibraries.AddRange(
            new string[]
            {
                Path.Combine(azureKinectSensorSDKPath, "amd64", "lib", "k4a.lib"),
                Path.Combine(azureKinectBodyTrackingSDKPath, "amd64", "lib", "k4abt.lib")
            });

        // Ensure that the DLLs are staged along with the executable
        /*
        RuntimeDependencies.Add("$(PluginDir)/Binaries/Win64/k4a.dll");
        RuntimeDependencies.Add("$(PluginDir)/Binaries/Win64/k4abt.dll");
        RuntimeDependencies.Add("$(PluginDir)/Binaries/Win64/depthengine_2_0.dll");
        */

        return true;
    }
}
